-------------------------------------------------------------------------------
-- ElvUI PvP Datatext By Lockslap (US, Bleeding Hollow)
-------------------------------------------------------------------------------+
local E, _, V, P, G = unpack(ElvUI)
local DT = E:GetModule("DataTexts")
local L = LibStub("AceLocale-3.0"):GetLocale("ElvUI_PvPDatatext", false)
local EP = LibStub("LibElvUIPlugin-1.0")
local Crayon = LibStub("LibCrayon-3.0")

local join = string.join
local ipairs = ipairs

local displayString = ""
local bgValues = {
	["2v2"] = 1,
	["3v3"] = 2,
	["5v5"] = 3,
	["rbg"] = 4,
}
local bgNames = {
	[1] = L["2v2 Rating"],
	[2] = L["3v3 Rating"],
	[3] = L["5v5 Rating"],
	[4] = L["RBG Rating"],
}

local function GetIndexByKey(needle, haystack)
	for i = 1, #haystack do
		if haystack[i] == needle then
			return i
		end
	end
	return false
end

local function GetDatatextInfo()
	local label, num, icon, earned, weeklyMax
	if E.db.pvpdt.display == "conq" or E.db.pvpdt.display == "honor" then
		label, num, icon = GetCurrencyInfo(E.db.pvpdt.display == "conq" and 390 or 392)
		return label, tostring(num), icon
	elseif E.db.pvpdt.display == "2v2" or E.db.pvpdt.display == "3v3" or E.db.pvpdt.display == "5v5" or E.db.pvpdt.display == "rbg" then
		num, _, _ = GetPersonalRatedInfo(bgValues[E.db.pvpdt.display])
		return bgNames[bgValues[E.db.pvpdt.display]], tostring(num), nil
	elseif E.db.pvpdt.display == "conqCap" then
		label, num, icon, earned, weeklyMax = GetCurrencyInfo(390)
		return L["Conquest Cap"], ("%d/%d"):format(earned, weeklyMax), icon
	elseif E.db.pvpdt.display == "honorCap" then
		label, num, icon = GetCurrencyInfo(392)
		return L["Honor Cap"], ("%d/4000"):format(num), icon
	elseif E.db.pvpdt.display == "resil" then
		return STAT_RESILIENCE, ("%.2f%%"):format(GetCombatRatingBonus(COMBAT_RATING_RESILIENCE_PLAYER_DAMAGE_TAKEN) + GetModResilienceDamageReduction()), nil
	elseif E.db.pvpdt.display == "power" then
		return STAT_PVP_POWER, ("%.2f%%"):format(GetCombatRatingBonus(CR_PVP_POWER)), nil
	end
	return false
end

local function OnEvent(self, event)
	if event == "PLAYER_ENTERING_WORLD" and PVPUIFrame == nil then
		LoadAddOn("Blizzard_PVPUI")
	end
	
	local text, num, icon = GetDatatextInfo()
	self.text:SetFormattedText(displayString, E.db.pvpdt.icon == true and ("|T%s:14:14:0:0:64:64:4:60:4:60|t "):format(icon) or "", text, num)
end

local function OnEnter(self)
	local ratedPVP = {[1] = "twos", [2] = "threes", [3] = "fives", [4] = "rbg"}
	local ratedHeaders = {[1] = L["2v2 Stats"], [2] = L["3v3 Stats"], [3] = L["5v5 Stats"], [4] = L["RBG Stats"]}
	local name, count, icon, earned, weeklyMax, totalMax	-- currency vars
	local rating, seasonBest, weeklyBest, seasonPlayed, seasonWon, weeklyPlayed, weeklyWon, cap	-- rated pvp stats
	local seasonLost, seasonWinPct, weeklyLost, weeklyWinPct
	local red, green, blue
	local weeklyRed, weeklyGreen, weeklyBlue, seasonRed, seasonGreen, seasonBlue
	local currencyString = "|T%s:14:14:0:0:64:64:4:60:4:60|t %s"
	local recordString = "%d-%d (%s)"
	local currencies = {390, 392}	-- honor and conquest points
	DT:SetupTooltip(self)
	
	-- currency list
	if E.db.pvpdt.tooltip.currency then
		DT.tooltip:AddLine(L["Currencies"])
		for _, value in ipairs(currencies) do
			name, count, icon, earned, weeklyMax, _, isDiscovered = GetCurrencyInfo(value)
			if isDiscovered then
				red, green, blue = Crayon:GetThresholdColor(value == 390 and earned / weeklyMax or count / 4000)
				DT.tooltip:AddDoubleLine(currencyString:format(icon, name), count, 1, 1, 1)
				DT.tooltip:AddDoubleLine(currencyString:format(icon, value == 390 and L["Conquest Cap"] or L["Honor Cap"]), ("%d/%d (%.1f%%)"):format(value == 390 and earned or count, value == 390 and weeklyMax or 4000, value == 390 and (earned / weeklyMax) * 100 or (count / 4000) * 100), 1, 1, 1, red, green, blue)
			end
		end
	end
	
	-- rated pvp
	for index, value in ipairs(ratedPVP) do
		if E.db.pvpdt.tooltip[value] and E.db.pvpdt.tooltip[value] == true then
			DT.tooltip:AddLine(" ")
			
			-- get the info, and do some calculations
			rating, seasonBest, weeklyBest, seasonPlayed, seasonWon, weeklyPlayed, weeklyWon, cap = GetPersonalRatedInfo(index)
			seasonLost = seasonPlayed - seasonWon
			weeklyLost = weeklyPlayed - weeklyWon
			
			-- calculate win percentages
			seasonWinPct = seasonPlayed > 0 and seasonWon / seasonPlayed or 0
			weeklyWinPct = weeklyPlayed > 0 and weeklyWon / weeklyPlayed or 0
			
			-- now get the colors
			seasonRed, seasonGreen, seasonBlue = Crayon:GetThresholdColor(seasonWinPct)
			weeklyRed, weeklyGreen, weeklyBlue = Crayon:GetThresholdColor(weeklyWinPct)
			
			DT.tooltip:AddLine(ratedHeaders[index])
			DT.tooltip:AddDoubleLine(L["Rating"], rating or 0, 1, 1, 1)
			DT.tooltip:AddDoubleLine(L["Weekly Best Rating"], weeklyBest or 0, 1, 1, 1)
			DT.tooltip:AddDoubleLine(L["Weekly Record"], recordString:format(weeklyWon or 0, weeklyLost or 0, ("%.0f%%"):format(weeklyWinPct * 100) or "0%"), 1, 1, 1, weeklyRed, weeklyGreen, weeklyBlue)
			DT.tooltip:AddDoubleLine(L["Season Best Rating"], seasonBest or 0, 1, 1, 1)
			DT.tooltip:AddDoubleLine(L["Season Record"], recordString:format(seasonWon or 0, seasonLost or 0, ("%.0f%%"):format(seasonWinPct * 100) or "0%"), 1, 1, 1, seasonRed, seasonGreen, seasonBlue)
			DT.tooltip:AddDoubleLine(L["Projected Cap"], cap, 1, 1, 1)
		end
	end
	
	-- pvp gear stats
	if E.db.pvpdt.tooltip.stats then
		DT.tooltip:AddLine(" ")
		DT.tooltip:AddLine(L["PvP Gear Stats"])
		
		-- item level
		local total, equipped = GetAverageItemLevel()
		DT.tooltip:AddDoubleLine(L["Item Level"], ("%d (%d %s)"):format(total, equipped, L["Equipped"]), 1, 1, 1)
	
		-- pvp resilience
		local resilRating = BreakUpLargeNumbers(GetCombatRating(COMBAT_RATING_RESILIENCE_PLAYER_DAMAGE_TAKEN))
		local ratingBonus = GetCombatRatingBonus(COMBAT_RATING_RESILIENCE_PLAYER_DAMAGE_TAKEN)
		local damageReduction = ratingBonus + GetModResilienceDamageReduction()
		DT.tooltip:AddDoubleLine(L["Resilience"], ("%s (+%.2f%%)"):format(resilRating, ratingBonus), 1, 1, 1)
		DT.tooltip:AddDoubleLine(L["Damage Reduction"], ("%.2f%%"):format(damageReduction), 1, 1, 1)
		
		-- pvp power
		local pvpPower = BreakUpLargeNumbers(GetCombatRating(CR_PVP_POWER))
		local pvpDamage = GetPvpPowerDamage()
		local pvpHealing = GetPvpPowerHealing()
		DT.tooltip:AddDoubleLine(L["Power"], pvpPower, 1, 1, 1)
		DT.tooltip:AddDoubleLine(pvpHealing > pvpDamage and L["Healing"] or L["Damage"], ("+%.2f%%"):format(pvpHealing > pvpDamage and pvpHealing or pvpDamage), 1, 1, 1)
	end
	
	-- make sure something gets added
	if not E.db.pvpdt.tooltip.currency and not E.db.pvpdt.tooltip.twos and not E.db.pvpdt.tooltip.threes and not E.db.pvpdt.tooltip.fives and not E.db.pvpdt.tooltip.rbg and not E.db.pvpdt.tooltip.stats then
		DT.tooltip:AddLine(L["Nothing to display."], 1, 0, 0)
	end
	
	if E.db.pvpdt.tooltip.hint then
		DT.tooltip:AddLine(" ")
		DT.tooltip:AddDoubleLine(L["Left Click"], E.db.pvpdt.leftClick == "curFrame" and L["Toggle Currency Frame"] or L["Toggle PVP UI"], 1, 1, 1)
		DT.tooltip:AddDoubleLine(L["Right Click"], E.db.pvpdt.rightClick == "curFrame" and L["Toggle Currency Frame"] or L["Toggle PVP UI"], 1, 1, 1)
	end
	
	DT.tooltip:Show()
end

local function OnClick(self, button)
	if button == "LeftButton" then
		if E.db.pvpdt.leftClick == "curFrame" then
			ToggleCharacter("TokenFrame")
		else
			if not PVPUIFrame then
				PVP_LoadUI()
			end	
			ToggleFrame(PVPUIFrame)
		end
	elseif button == "RightButton" then
		if E.db.pvpdt.rightClick == "curFrame" then
			ToggleCharacter("TokenFrame")
		else
			if not PVPUIFrame then
				PVP_LoadUI()
			end
			ToggleFrame(PVPUIFrame)
		end
	end
end

local function ValueColorUpdate(hex, r, g, b)
	displayString = join("", "%s", "|cffffffff%s:|r", " ", hex, "%s|r")
end
E["valueColorUpdateFuncs"][ValueColorUpdate] = true

P["pvpdt"] = {
	["display"] = "conq",
	["icon"] = true,
	["leftClick"] = "curFrame",
	["rightClick"] = "pvpFrame",
	["tooltip"] = {
		["currency"] = true,
		["twos"] = true,
		["threes"] = true,
		["fives"] = true,
		["rbg"] = true,
		["stats"] = true,
		["hint"] = true,
	},
}

local function InjectOptions()
	if not E.Options.args.lockslap then
		E.Options.args.lockslap = {
			type = "group",
			order = -2,
			name = L["Plugins by |cff9382c9Lockslap|r"],
			args = {
				thanks = {
					type = "description",
					order = 1,
					name = L["Thanks for using and supporting my work!  -- |cff9382c9Lockslap|r\n\n|cffff0000If you find any bugs, or have any suggestions for any of my addons, please open a ticket at that particular addon's page on CurseForge."],
				},
			},
		}
	elseif not E.Options.args.lockslap.args.thanks then
		E.Options.args.lockslap.args.thanks = {
			type = "description",
			order = 1,
			name = L["Thanks for using and supporting my work!  -- |cff9382c9Lockslap|r\n\n|cffff0000If you find any bugs, or have any suggestions for any of my addons, please open a ticket at that particular addon's page on CurseForge."],
		}
	end
	
	E.Options.args.lockslap.args.pvpdt = {
		type = "group",
		name = L["PvP Datatext"],
		get = function(info) return E.db.pvpdt[info[#info]] end,
		set = function(info, value) E.db.pvpdt[info[#info]] = value; DT:LoadDataTexts() end,
		args = {
			display = {
				type = "select",
				order = 1,
				name = L["Display"],
				desc = L["What to display on the datatext."],
				values = {
					["conq"] = GetCurrencyInfo(390),
					["honor"] = GetCurrencyInfo(392),
					["2v2"] = L["2v2 Rating"],
					["3v3"] = L["3v3 Rating"],
					["5v5"] = L["5v5 Rating"],
					["rbg"] = L["RBG Rating"],
					["conqCap"] = L["Conquest Cap"],
					["honorCap"] = L["Honor Cap"],
					["resil"] = STAT_RESILIENCE,
					["power"] = STAT_PVP_POWER,
				},
			},
			icon = {
				type = "toggle",
				order = 2,
				width = "double",
				name = L["Include Icon"],
				desc = L["Include icon on the datatext.  Must have Conquest/Honor Points or Caps selected."],
				disabled = function()
					if E.db.pvpdt.display == "2v2" or E.db.pvpdt.display == "3v3" or E.db.pvpdt.display == "5v5" or E.db.pvpdt.display == "rbg" or E.db.pvpdt.display == "resil" or E.db.pvpdt.display == "power" then
						return true
					else
						return false
					end
				end,
			},
			leftClick = {
				type = "select",
				order = 3,
				name = L["Left Click Action"],
				desc = L["What to do when the datatext is left clicked."],
				values = {
					["curFrame"] = L["Toggle Currency Frame"],
					["pvpFrame"] = L["Toggle PvP Frame"],
				},
			},
			rightClick = {
				type = "select",
				order = 4,
				name = L["Right Click Action"],
				desc = L["What to do when the datatext is right clicked."],
				values = {
					["curFrame"] = L["Toggle Currency Frame"],
					["pvpFrame"] = L["Toggle PvP Frame"],
				},
			},
			tooltip = {
				type = "group",
				order = -1,
				name = L["Tooltip Options"],
				guiInline = true,
				get = function(info) return E.db.pvpdt.tooltip[info[#info]] end,
				set = function(info, value) E.db.pvpdt.tooltip[info[#info]] = value; DT:LoadDataTexts() end,
				args = {
					currency = {
						type = "toggle",
						order = 1,
						name = L["Show PvP Currencies"],
						desc = L["Show honor and conquest quantities on the tooltip."],
					},
					twos = {
						type = "toggle",
						order = 2,
						name = L["Show 2v2 Stats"],
						desc = L["Show your 2v2 arena stats."],
					},
					threes = {
						type = "toggle",
						order = 3,
						name = L["Show 3v3 Stats"],
						desc = L["Show your 3v3 arena stats."],
					},
					fives = {
						type = "toggle",
						order = 4,
						name = L["Show 5v5 Stats"],
						desc = L["Show your 5v5 arena stats."],
					},
					rbg = {
						type = "toggle",
						order = 5,
						name = L["Show RBG Stats"],
						desc = L["Show your RBG stats."],
					},
					stats = {
						type = "toggle",
						order = 6,
						name = L["Show PvP Stats"],
						desc = L["Show your PvP stats, resilience, power, and item level."],
					},
					hint = {
						type = "toggle",
						order = 7,
						name = L["Show Hint"],
						desc = L["Show hint in the tooltip."],
					},
				},
			},
		},
	}
end

EP:RegisterPlugin(..., InjectOptions)
DT:RegisterDatatext("PvP", {"PLAYER_ENTERING_WORLD", "CHAT_MSG_CURRENCY", "CHAT_MSG_LOOT"}, OnEvent, nil, OnClick, OnEnter)